package com.baidu.cloud.videoplayer.demo.service;

import com.baidu.cloud.videoplayer.demo.R;
import com.baidu.cloud.videoplayer.demo.bar.AdvancedMediaController;
import com.baidu.cloud.videoplayer.demo.helper.CommonUtils;
import com.baidu.cloud.videoplayer.demo.receiver.NotificationClickReceiver;

import android.annotation.TargetApi;
import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.BitmapFactory;
import android.os.Binder;
import android.os.Build;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;
import android.widget.RemoteViews;
import androidx.core.app.NotificationCompat;

public class BackgroundPlayService extends Service {
    private static final String TAG = "BackgroundPlayService";

    private static final int INTERNAL_TIME = 1000;

    private AdvancedMediaController mediaController;
    private String title;

    private static Notification notification;
    private static RemoteViews remoteViews;
    private final int NOTIFICATION_ID = 1;
    private static NotificationManager manager;

    private BgPlayReceiver bgPlayReceiver;
    private static final String PLAY = "play";
    private static final String PAUSE = "pause";
    private static final String CLOSE = "close";

    public class BackgroundPlayBinder extends Binder {
        public BackgroundPlayService getService() {
            return BackgroundPlayService.this;
        }
    }

    @Override
    public IBinder onBind(Intent intent) {
//        super.onBind(intent);
        return new BackgroundPlayBinder();
    }

    @Override
    public void onCreate() {
        Log.d(TAG, "onCreate");
        super.onCreate();
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Log.d(TAG, "onStartCommand");
        initRemoteViews();
        startMyOwnForeground();
        registerBgPlayReceiver();
        return Service.START_STICKY;
    }

    private void startMyOwnForeground() {
        manager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            String channelId = "bgplay_control";
            String channelName = "后台播放控制";
            int importance = NotificationManager.IMPORTANCE_HIGH;
            createNotificationChannel(channelId, channelName, importance);
        }

        // 点击整个通知时发送广播
        Intent intent = new Intent(getApplicationContext(), NotificationClickReceiver.class);
        PendingIntent pendingIntent = PendingIntent.getBroadcast(getApplicationContext(), 0,
                intent, PendingIntent.FLAG_UPDATE_CURRENT);

        // 初始化通知
        notification = new NotificationCompat.Builder(this, "bgplay_control")
                .setContentIntent(pendingIntent)
                .setWhen(System.currentTimeMillis())
                .setSmallIcon(R.drawable.icon_music)
                .setLargeIcon(BitmapFactory.decodeResource(getResources(), R.drawable.icon_music))
                .setCustomContentView(remoteViews)
                .setVisibility(NotificationCompat.VISIBILITY_PUBLIC)
                .setAutoCancel(false)
                .setOnlyAlertOnce(true)
                .setOngoing(true)
                .build();
        startForeground(NOTIFICATION_ID, notification);
    }

    public void setMediaController(AdvancedMediaController mediaController) {
        this.mediaController = mediaController;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public void togglePlayPause() {
        if (mediaController != null) {
            mediaController.togglePlayPause();
        }
        updateNotificationShow();
    }

    public void closeNotification(boolean pausePlay) {
        if (mediaController != null) {
            if (pausePlay && mediaController.isPlaying()) {
                mediaController.togglePlayPause();
            }
        }
        mHandler.removeCallbacksAndMessages(null);
        manager.cancel(NOTIFICATION_ID);
    }

    private void initRemoteViews() {
        remoteViews = new RemoteViews(this.getPackageName(), R.layout.layout_notification);

        Intent intentPlay = new Intent(PLAY);
        PendingIntent playPendingIntent = PendingIntent.getBroadcast(this, 0, intentPlay, 0);
        remoteViews.setOnClickPendingIntent(R.id.btn_notification_play, playPendingIntent);

        Intent intentClose = new Intent(CLOSE);
        PendingIntent closePendingIntent = PendingIntent.getBroadcast(this, 0, intentClose, 0);
        remoteViews.setOnClickPendingIntent(R.id.btn_notification_close, closePendingIntent);

    }

    public void updateNotificationShow() {
        if (mediaController != null) {
            if (mediaController.isPlaying()) {
                remoteViews.setImageViewResource(R.id.btn_notification_play,
                        R.drawable.toggle_btn_pause);
            } else {
                remoteViews.setImageViewResource(R.id.btn_notification_play,
                        R.drawable.btn_play_normal);
            }
            remoteViews.setTextViewText(R.id.tv_notification_title, title);
        }
        manager.notify(NOTIFICATION_ID, notification);
    }

    @TargetApi(Build.VERSION_CODES.O)
    private void createNotificationChannel(String channelId, String channelName, int importance) {
        NotificationChannel channel = new NotificationChannel(channelId, channelName, importance);
        channel.enableLights(false);
        channel.enableVibration(false);
        channel.setVibrationPattern(new long[]{0});
        channel.setSound(null, null);
        if (manager != null) {
            manager.createNotificationChannel(channel);
        }
    }

    private void registerBgPlayReceiver() {
        bgPlayReceiver = new BgPlayReceiver();
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(PLAY);
        intentFilter.addAction(CLOSE);
        registerReceiver(bgPlayReceiver, intentFilter);
    }

    public class BgPlayReceiver extends BroadcastReceiver {

        @Override
        public void onReceive(Context context, Intent intent) {
            switch (intent.getAction()) {
                case PLAY:
                    togglePlayPause();
                    break;
                case CLOSE:
                    closeNotification(true);
                    stopForeground(true);
                    break;
                default:
                    break;
            }
        }
    }

    private final Handler mHandler = new Handler(new Handler.Callback() {
        @Override
        public boolean handleMessage(Message message) {
            // 更新进度
            updateProgress();
            return true;
        }
    });

    public void updateProgress() {
        if (mediaController != null) {
            int progress = mediaController.getCurrentPosition();
            int duration = mediaController.getDuration();
            remoteViews.setTextViewText(R.id.tv_notification_position, String.format("%s/%s",
                    CommonUtils.stringForTime(progress),
                    CommonUtils.stringForTime(duration)));
            updateNotificationShow();
        }
        mHandler.sendEmptyMessageDelayed(100, INTERNAL_TIME);
    }

    @Override
    public void onDestroy() {
        Log.d(TAG, "onDestroy");
        super.onDestroy();
        if (bgPlayReceiver != null) {
            unregisterReceiver(bgPlayReceiver);
        }
        stopForeground(true);
    }
}