package com.baidu.cloud.videoplayer.widget;

import android.annotation.TargetApi;
import android.content.Context;
import android.graphics.SurfaceTexture;
import android.os.Build;
import android.util.AttributeSet;
import android.util.Log;
import android.view.Surface;
import android.view.TextureView;
import android.view.View;
import android.view.accessibility.AccessibilityEvent;
import android.view.accessibility.AccessibilityNodeInfo;

import com.baidu.cloud.media.player.IMediaPlayer;
import com.baidu.cloud.media.player.render.record.IRecordListener;
import com.baidu.cloud.media.player.render.record.RecordConstants;
import com.baidu.cloud.media.player.render.record.RecordController;
import com.baidu.cloud.media.player.render.texturerender.VideoTextureRender;
import com.baidu.cloud.videoplayer.demo.C;
import com.baidu.cloud.videoplayer.demo.helper.CommonUtils;

import java.lang.ref.WeakReference;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
public class TextureRenderView extends TextureView implements IRenderView {
    private static final String TAG = "TextureRenderView";
    private MeasureHelper mMeasureHelper;

    private SurfaceTexture mNewSurfaceTexture;

    private VideoTextureRender mVideoRender;
    private RecordController mRecordController;

    public TextureRenderView(Context context) {
        super(context);
        initView(context);
    }

    public TextureRenderView(Context context, AttributeSet attrs) {
        super(context, attrs);
        initView(context);
    }

    public TextureRenderView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initView(context);
    }

    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    public TextureRenderView(Context context, AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        initView(context);
    }

    private void initView(Context context) {
        mMeasureHelper = new MeasureHelper(this);
        mSurfaceCallback = new SurfaceCallback(this);
        setSurfaceTextureListener(mSurfaceCallback);
    }

    @Override
    public View getView() {
        return this;
    }

    @Override
    public boolean shouldWaitForResize() {
        return false;
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        Log.d(TAG, "onDetachedFromWindow");
//        release(); // release surface when detach. if uncomment, not support float window
    }

    // --------------------
    // Layout & Measure
    // --------------------
    @Override
    public void setVideoSize(int videoWidth, int videoHeight) {
        if (videoWidth > 0 && videoHeight > 0) {
            mMeasureHelper.setVideoSize(videoWidth, videoHeight);
            requestLayout();
        }
    }

    @Override
    public void setVideoSampleAspectRatio(int videoSarNum, int videoSarDen) {
        if (videoSarNum > 0 && videoSarDen > 0) {
            mMeasureHelper.setVideoSampleAspectRatio(videoSarNum, videoSarDen);
            requestLayout();
        }
    }

    @Override
    public void setVideoRotation(int degree) {
        mMeasureHelper.setVideoRotation(degree);
        setRotation(degree);
    }

    @Override
    public void setAspectRatio(int aspectRatio) {
        mMeasureHelper.setAspectRatio(aspectRatio);
        requestLayout();
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        mMeasureHelper.doMeasure(widthMeasureSpec, heightMeasureSpec);
        setMeasuredDimension(mMeasureHelper.getMeasuredWidth(), mMeasureHelper.getMeasuredHeight());
    }

    private int mMediaPlayerCode = 0;

    public void setCurrentMediaPlayerCode(int currentCode) {
        mMediaPlayerCode = currentCode;
    }

    public int getCurrentMediaPlayerCode() {
        return mMediaPlayerCode;
    }

    private SurfaceTexture lastSurfaceTexture;

    public SurfaceTexture getLastSurfaceTexture() {
        return lastSurfaceTexture;
    }

    public void setLastSurfaceTexture(SurfaceTexture surfaceTexture) {
        lastSurfaceTexture = surfaceTexture;
    }

    // --------------------
    // TextureViewHolder
    // --------------------

    public IRenderView.ISurfaceHolder getSurfaceHolder() {
        return new InternalSurfaceHolder(this);
    }

    private final class InternalSurfaceHolder implements IRenderView.ISurfaceHolder {
        private TextureRenderView mTextureRenderView;

        public InternalSurfaceHolder(TextureRenderView textureView) {
            mTextureRenderView = textureView;
        }

        @TargetApi(Build.VERSION_CODES.JELLY_BEAN)
        public void bindToMediaPlayer(IMediaPlayer mp) {
            if (mp == null || mTextureRenderView.getSurfaceTexture() == null) {
                // if reSetRender is invoked twice during one playing, surfaceTexture maybe null
                return;
            }
            // Tricky: 由 Object 类定义的 hashCode 方法确实会针对不同的对象返回不同的整数。
            // （这一般是通过将该对象的内部地址转换成一个整数来实现的，但是 JavaTM 编程语言不需要这种实现技巧。）
            if (mp.hashCode() != mTextureRenderView.getCurrentMediaPlayerCode()) {
                // new player
                mp.setSurface(openSurface());
            } else {
                // old player
                if (!mTextureRenderView.getLastSurfaceTexture().equals(mTextureRenderView.getSurfaceTexture())) {
                    mTextureRenderView.setSurfaceTexture(mTextureRenderView.getLastSurfaceTexture());
                }
            }
            mTextureRenderView.setCurrentMediaPlayerCode(mp.hashCode());
        }

        @Override
        public IRenderView getRenderView() {
            return mTextureRenderView;
        }

        @Override
        public Surface openSurface() {
            if (mVideoRender != null) {
                mNewSurfaceTexture = mVideoRender.getVideoTexture();
            } else {
                // TextureView 默认内部创建
                mNewSurfaceTexture = mTextureRenderView.getSurfaceTexture();
            }

            return new Surface(mNewSurfaceTexture);
        }
    }

    // SurfaceHolder.Callback
    // -------------------------
    // -------------------------

    @Override
    public void addRenderCallback(IRenderCallback callback) {
        mSurfaceCallback.addRenderCallback(callback);
    }

    @Override
    public void removeRenderCallback(IRenderCallback callback) {
        mSurfaceCallback.removeRenderCallback(callback);
    }

    @TargetApi(16)
    @Override
    public void release() {
        if (lastSurfaceTexture != null) {
            if (this.isAvailable()) {
                // let textureview to release surfacetexture
                mSurfaceCallback.setOwnSurfaceTexture(true);
            } else {
                lastSurfaceTexture.release();
                lastSurfaceTexture = null;
            }
        }
    }

    @Override
    public void destroy() {
        Log.d(TAG, "textureRenderView destroy()");
        if (mVideoRender != null) {
            mVideoRender.onDestroy();
            mVideoRender = null;
        }

        if (mRecordController != null) {
            mRecordController.onDestroy();
            mRecordController = null;
        }
    }

    @Override
    public void startRecord(String filePath) {
        if (mRecordController == null) {
            mRecordController = new RecordController();
            mRecordController.init(mNewSurfaceTexture, RecordConstants.VIDEO_ENCODE_FRAME_RATE,
                    RecordConstants.DEFAULT_BIT_RATE_GTE_API18);

            if (mVideoRender != null) {
                mVideoRender.setRenderList(mRecordController.getRenderCallbackList());
            }
        }

        mRecordController.setRecordSize(mWidth, mHeight);
        mRecordController.startRecord(filePath);
    }

    @Override
    public void setRecordListener(IRecordListener listener) {
        if (mRecordController != null) {
            mRecordController.setRecordListener(listener);
        }
    }

    @Override
    public void stopRecord() {
        if (mRecordController != null) {
            mRecordController.stopRecord();
        }
    }

    @Override
    public void onUpdateAudioData(byte[] bytes, int byteSize) {
        if (mRecordController != null) {
            mRecordController.updateExternalAudioData(bytes, byteSize);
        }
    }

    private SurfaceTexture mSurfaceTexture;
    private SurfaceCallback mSurfaceCallback;
    private int mWidth;
    private int mHeight;

    private final class SurfaceCallback implements TextureView.SurfaceTextureListener {

        private boolean mIsFormatChanged;

        private volatile boolean mOwnSurfaceTexture = false;
//        private boolean mWillDetachFromWindow = false;
//        private boolean mDidDetachFromWindow = false;

        private WeakReference<TextureRenderView> mWeakRenderView;
        private Map<IRenderCallback, Object> mRenderCallbackMap = new ConcurrentHashMap<IRenderCallback, Object>();

        public SurfaceCallback(TextureRenderView renderView) {
            mWeakRenderView = new WeakReference<TextureRenderView>(renderView);
        }

        public void setOwnSurfaceTexture(boolean ownSurfaceTexture) {
            mOwnSurfaceTexture = ownSurfaceTexture;
        }

        public void addRenderCallback(IRenderCallback callback) {
            mRenderCallbackMap.put(callback, callback);

            ISurfaceHolder surfaceHolder = null;
            if (mSurfaceTexture != null) {
                if (surfaceHolder == null) {
                    surfaceHolder = new InternalSurfaceHolder(mWeakRenderView.get());
                }
                callback.onSurfaceCreated(surfaceHolder, mWidth, mHeight);
            }

            if (mIsFormatChanged) {
                if (surfaceHolder == null) {
                    surfaceHolder = new InternalSurfaceHolder(mWeakRenderView.get());
                }
                callback.onSurfaceChanged(surfaceHolder, 0, mWidth, mHeight);
            }
        }

        public void removeRenderCallback(IRenderCallback callback) {
            mRenderCallbackMap.remove(callback);
        }

        @TargetApi(16)
        @Override
        public void onSurfaceTextureAvailable(SurfaceTexture surface, int width, int height) {

            mSurfaceTexture = surface;
            if (mWeakRenderView.get() == null) {
                // should never come in
                Log.d(TAG, "!!!!!Too bad, textureview in callback is released. function will not work normally");
            } else {
                // only set once
                if (mWeakRenderView.get().getLastSurfaceTexture() == null) {
                    mWeakRenderView.get().setLastSurfaceTexture(mSurfaceTexture);
                }
            }

            // 创建render 和 SurfaceTexture
            // TODO hdr和暂时不能和录制共同使用
            if (CommonUtils.getSmartVisionType(getContext()) != C.SMARTVISION_HDR) {
                if (mVideoRender == null) {
                    mVideoRender = new VideoTextureRender(mSurfaceTexture, width, height);
                    mVideoRender.onInitCreate();
                    mVideoRender.setVideoSize(width, height);
                }
            }

            mIsFormatChanged = false;
            mWidth = width;
            mHeight = height;

            Log.d(TAG, "onSurfaceTextureAvailable() " + width + "-" + height);

            ISurfaceHolder surfaceHolder = new InternalSurfaceHolder(mWeakRenderView.get());
            for (IRenderCallback renderCallback : mRenderCallbackMap.keySet()) {
                renderCallback.onSurfaceCreated(surfaceHolder, 0, 0);
            }
        }

        @Override
        public void onSurfaceTextureSizeChanged(SurfaceTexture surface, int width, int height) {
            mSurfaceTexture = surface;
            mIsFormatChanged = true;
            mWidth = width;
            mHeight = height;

            Log.d(TAG, "onSurfaceTextureSizeChanged() " + width + "-" + height);

            if (mVideoRender != null) {
                mVideoRender.setVideoSize(width, height);
            }

            if (mRecordController != null) {
                mRecordController.setRecordSize(width, height);
            }

            ISurfaceHolder surfaceHolder = new InternalSurfaceHolder(mWeakRenderView.get());
            for (IRenderCallback renderCallback : mRenderCallbackMap.keySet()) {
                renderCallback.onSurfaceChanged(surfaceHolder, 0, width, height);
            }
        }

        @Override
        public boolean onSurfaceTextureDestroyed(SurfaceTexture surface) {
            mSurfaceTexture = surface;
            mIsFormatChanged = false;
            mWidth = 0;
            mHeight = 0;

            Log.d(TAG, "onSurfaceTextureDestroyed()");

            if (mRecordController != null && mRecordController.isRecording()) {
                mRecordController.stopRecord();
            }
            if (mVideoRender != null) {
                mVideoRender.onDestroy();
                mVideoRender = null;
            }

            ISurfaceHolder surfaceHolder = new InternalSurfaceHolder(mWeakRenderView.get());
            for (IRenderCallback renderCallback : mRenderCallbackMap.keySet()) {
                renderCallback.onSurfaceDestroyed(surfaceHolder);
            }

            return mOwnSurfaceTexture;
        }

        @Override
        public void onSurfaceTextureUpdated(SurfaceTexture surface) {
            // Logger.e(TAG, "onSurfaceTextureUpdated()");
        }

    }

    // --------------------
    // Accessibility
    // --------------------

    @Override
    public void onInitializeAccessibilityEvent(AccessibilityEvent event) {
        super.onInitializeAccessibilityEvent(event);
        event.setClassName(TextureRenderView.class.getName());
    }

    @Override
    public void onInitializeAccessibilityNodeInfo(AccessibilityNodeInfo info) {
        super.onInitializeAccessibilityNodeInfo(info);
        info.setClassName(TextureRenderView.class.getName());
    }
}
